package com.dodo.leancloud;

import android.util.Log;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.avos.avoscloud.AVCloudQueryResult;
import com.avos.avoscloud.AVException;
import com.avos.avoscloud.AVFile;
import com.avos.avoscloud.AVObject;
import com.avos.avoscloud.AVQuery;
import com.avos.avoscloud.CloudQueryCallback;
import com.avos.avoscloud.DeleteCallback;
import com.avos.avoscloud.FindCallback;

import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static com.avos.avoscloud.AVObject.CREATED_AT;
import static com.avos.avoscloud.AVObject.OBJECT_ID;
import static com.avos.avoscloud.AVObject.UPDATED_AT;


/**
 * Created by 刘铎 on 2019/4/8+
 */
public class AVObjectPlugin {
    private static final String TAG = AVObjectPlugin.class.getSimpleName();
    public static String OBJECT_PREFIX = "object_";
    public static String FILE_PREFIX = "file_";

    public static String AVFILE_CLASSNAME = "AVFile";
    public static String AVFILE_NAME = "name";
    public static String AVFILE_URL = "url";
    public static String AVFILE_LOCALPATH = "localPath";

    public String className;

    public Map<String, Object> _map = new HashMap<>();

    public static AVObjectPlugin parseFormJson(String json) {
        AVObjectPlugin avObjectPlugin = JSON.parseObject(json, AVObjectPlugin.class);
        if (avObjectPlugin == null) {
            return null;
        }
        Map<String, Object> map = avObjectPlugin._map;
        for (String key : map.keySet()) {
            if (key.startsWith(OBJECT_PREFIX) || key.startsWith(FILE_PREFIX)) {
                String innerJson = (String) map.get(key);
                map.put(key, parseFormJson(innerJson));
                continue;
            }
        }
        return avObjectPlugin;
    }

    public static AVObjectPlugin parseFormAVObject(AVObject avObject, Map<String, Object> _map) {
        if (_map == null || avObject == null) {
            return null;
        }
        AVObjectPlugin avObjectPlugin = new AVObjectPlugin();
        avObjectPlugin.className = avObject.getClassName();
        Map<String, Object> targetMap = avObjectPlugin._map;
        for (String key : _map.keySet()) {
            if (key.startsWith(OBJECT_PREFIX)) {
                AVObjectPlugin innerAVObjectPlugin = (AVObjectPlugin) _map.get(key);
                if (innerAVObjectPlugin == null) {
                    targetMap.put(key, null);
                    continue;
                }
                Map<String, Object> innerMap = innerAVObjectPlugin._map;

                AVObject value = (AVObject) avObject.get(key);
                targetMap.put(key, parseFormAVObject(value, innerMap));
                continue;
            }
            if (key.startsWith(FILE_PREFIX)) {
                AVFile value = (AVFile) avObject.get(key);

                targetMap.put(key, parseFormAVFile(value));
                continue;
            }
            targetMap.put(key, avObject.get(key));
        }
        targetMap.put(OBJECT_ID, avObject.getObjectId());
        Date createdAt = avObject.getCreatedAt();
        if (createdAt != null) {
            targetMap.put(CREATED_AT, createdAt.getTime());
        }
        Date updatedAt = avObject.getUpdatedAt();
        if (updatedAt != null) {
            targetMap.put(UPDATED_AT, updatedAt.getTime());
        }
        return avObjectPlugin;
    }

    private static AVObjectPlugin parseFormAVFile(AVFile avFile) {
        if (avFile == null) {
            return null;
        }
        AVObjectPlugin avObjectPlugin = new AVObjectPlugin();
        avObjectPlugin.className = AVFILE_CLASSNAME;
        Map<String, Object> targetMap = avObjectPlugin._map;
        targetMap.put(OBJECT_ID, avFile.getObjectId());
        targetMap.put(AVFILE_NAME, avFile.getName());
        targetMap.put(AVFILE_URL, avFile.getUrl());
        return avObjectPlugin;
    }

    public String toJson() {
        return JSON.toJSONString(this, SerializerFeature.WriteMapNullValue);
    }

    public AVObject buildAVObject() {
        AVObject avObject = new AVObject(className);
        for (String key : _map.keySet()) {
            if (key.startsWith(OBJECT_PREFIX)) {
                AVObjectPlugin avObjectPlugin = (AVObjectPlugin) _map.get(key);
                if (avObjectPlugin != null) {
                    avObject.put(key, avObjectPlugin.buildAVObject());
                }
                continue;
            }
            if (key.startsWith(FILE_PREFIX)) {
                AVObjectPlugin avObjectPlugin = (AVObjectPlugin) _map.get(key);

                if (avObjectPlugin != null) {
                    avObject.put(key, avObjectPlugin.buildAVFile());
                }
                continue;
            }
            avObject.put(key, _map.get(key));
        }
        return avObject;
    }

    public AVFile buildAVFile() {
        String localPath = (String) _map.get(AVFILE_LOCALPATH);
        String name = (String) _map.get(AVFILE_NAME);
        AVFile avFile = null;
        try {
            avFile = AVFile.withAbsoluteLocalPath(name, localPath);
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
        return avFile;
    }

    public AVObject buildAVObjectForDelete() {
        String objectId = (String) _map.get(OBJECT_ID);
        AVObject avObject = AVObject.createWithoutData(className, objectId);
        return avObject;
    }

    public AVQuery<AVObject> buildAVQuery() {
        AVQuery<AVObject> avQuery = new AVQuery<>(className);
        for (String key : _map.keySet()) {
            if (key.startsWith(OBJECT_PREFIX) || key.startsWith(FILE_PREFIX)) {
                avQuery.include(key);
                continue;
            }
            Object value = _map.get(key);
            if (value != null) {
                avQuery.whereEqualTo(key, value);
            }
        }
        return avQuery;
    }

    public void saveInBackground(final SaveCallback callback) {
        Log.d(TAG, "thread:"+Thread.currentThread().getName());
        buildAVObject()
                .saveInBackground(new com.avos.avoscloud.SaveCallback() {
                    @Override
                    public void done(AVException e) {
                        if (e == null) {
                            callback.onSave(true);
                            Log.d(TAG, " save success!");
                        } else {
                            e.printStackTrace();
                            callback.onSave(false);
                        }
                    }
                });
    }

    public void findInBackground(final FindCallback callback) {
        buildAVQuery()
                .findInBackground(new com.avos.avoscloud.FindCallback<AVObject>() {
                    @Override
                    public void done(List<AVObject> avObjects, AVException e) {
                        if (e == null && avObjects != null) {
                            List<String> result = new ArrayList<>();
                            for (AVObject avObject : avObjects) {
                                result.add(parseFormAVObject(avObject, _map).toJson());
                            }
                            callback.onFind(result);
                            Log.d(TAG, avObjects.toString());
                        } else {
                            callback.onFind(null);
                            e.printStackTrace();
                        }
                    }
                });
    }

    public void deleteInBackground(final DeleteCallback callback) {
        buildAVObjectForDelete()
                .deleteInBackground(new com.avos.avoscloud.DeleteCallback() {
                    @Override
                    public void done(AVException e) {
                        if (e == null) {
                            callback.onDelete(true);
                        } else {
                            callback.onDelete(false);
                            e.printStackTrace();
                        }
                    }
                });

    }

    public void doCloudQueryInBackground(String cql, final CQLCallback callback) {
        AVQuery.doCloudQueryInBackground(cql, new CloudQueryCallback<AVCloudQueryResult>() {
            @Override
            public void done(AVCloudQueryResult avCloudQueryResult, AVException e) {
                if (e == null) {
                    List<String> result = new ArrayList<>();

                    if (avCloudQueryResult != null && avCloudQueryResult.getResults() != null) {
                        for (AVObject avObject : avCloudQueryResult.getResults()) {
                            result.add(parseFormAVObject(avObject, _map).toJson());
                        }
                    }

                    callback.onCQL(result);
                    Log.d(TAG, avCloudQueryResult.toString());
                } else {
                    callback.onCQL(null);
                    e.printStackTrace();
                }
            }
        });
    }

    @Override
    public String toString() {
        return toJson();
    }

    public interface SaveCallback {
        void onSave(boolean isSuccess);
    }

    public interface FindCallback {
        void onFind(List<String> jsons);
    }

    public interface DeleteCallback {
        void onDelete(boolean isSuccess);
    }

    public interface CQLCallback {
        void onCQL(List<String> jsons);
    }
}
