import 'dart:async';

import 'package:flutter/services.dart';
import 'package:leancloud/avobject.dart';

///由于leancloud不支持dart
///作为简单版本的平台插件使用
class Leancloud {
  static const MethodChannel _channel = const MethodChannel('leancloud');

  ///初始化leancloud
  ///在调用所有其他方法之前使用
  ///请在主线程中使用
  static Future initialize(
    String appID,
    String appKey,
  ) async {
    assert(appID != null && appID.isNotEmpty);
    assert(appKey != null && appKey.isNotEmpty);

    await _channel.invokeMethod(
      'initialize',
      <String, dynamic>{
        'appID': appID,
        'appKey': appKey,
      },
    );
    return;
  }

  ///注册并返回结果
  ///如果为true表示成功
  ///如果为false表示失败
  static Future<bool> signUp(
    String username,
    String password,
  ) async {
    assert(username != null && username.isNotEmpty);
    assert(password != null && password.isNotEmpty);

    bool result = await _channel.invokeMethod(
      'signUp',
      <String, dynamic>{
        'username': username,
        'password': password,
      },
    );

    return result;
  }

  ///登录并返回roles列表
  ///如果为null表示登录失败
  ///如果size==0表示为登录成功但是没有设置role
  static Future<List<String>> login(
    String username,
    String password,
  ) async {
    assert(username != null && username.isNotEmpty);
    assert(password != null && password.isNotEmpty);

    List<dynamic> invokes = await _channel.invokeMethod(
      'login',
      <String, dynamic>{
        'username': username,
        'password': password,
      },
    );
    if (invokes == null) {
      return null;
    }
    List<String> result = List();
    for (dynamic invoke in invokes) {
      result.add(invoke);
    }
    return result;
  }

  ///登出
  static Future logout() async {
    await _channel.invokeMethod('logout');
    return;
  }

  ///上传一个AVObject
  ///如果为true表示成功
  ///如果为false表示失败
  static Future<bool> putAVObject(AVObject object) async {
    assert(AVObject != null);

    bool result = await _channel.invokeMethod(
      'putAVObject',
      <String, dynamic>{
        'json': object.toJson(),
      },
    );

    return result;
  }

  ///查询AVObject
  ///AVObject可以通过put来设置需要查询的字段，put不为空的值为查询参数
  ///如果为null表示失败
  static Future<List<AVObject>> getAVObject(AVObject object) async {
    assert(AVObject != null);

    List<dynamic> jsons = await _channel.invokeMethod(
      'getAVObject',
      <String, dynamic>{
        'json': object.toJson(),
      },
    );

    if (jsons == null) {
      return null;
    }

    List<AVObject> result = new List();
    for (var json in jsons) {
      result.add(AVObject.fromJson(json));
    }
    return result;
  }

  ///删除AVObject AVObject必须通过查询获得objectId
  ///如果为true表示成功
  ///如果为false表示失败
  static Future<bool> deleteAVObject(AVObject object) async {
    assert(AVObject != null);

    bool result = await _channel.invokeMethod(
      'deleteAVObject',
      <String, dynamic>{
        'json': object.toJson(),
      },
    );

    return result;
  }

  ///执行CQL语句
  ///AVObject主要用来控制返回的json携带字段
  ///如果是查询可以获得列表数据 AVObject可以通过put来设置需要查询的字段，put不为空的值为查询参数
  ///如果为null表示失败
  static Future<List<AVObject>> doCQLQuery(AVObject object,String cql) async {
    assert(AVObject != null);

    List<dynamic> jsons = await _channel.invokeMethod(
      'doCQLQuery',
      <String, dynamic>{
        'json': object.toJson(),
        'cql': cql,
      },
    );

    if (jsons == null) {
      return null;
    }

    List<AVObject> result = new List();
    for (var json in jsons) {
      result.add(AVObject.fromJson(json));
    }
    return result;
  }
}
